/* $Id: bonuspack.cpp,v 1.10 2005/06/28 13:55:17 chfreund Exp $ */

#include  "bonuspack.hpp"
#include  "avatar.hpp"

/**********************************************************/

BonusPack::BonusPack( World *const worldPointer )
         : m_bag( NULL ),
           m_worldPointer( worldPointer )
{
	m_bag = NEW Bonus* [Bonus::getNumTypes()];
	for( int i = 0; i < Bonus::getNumTypes(); i++ )  m_bag[i] = NULL;
	m_bag -= FIRST_BONUS;
}

/**********************************************************/

BonusPack::~BonusPack()
{
	m_bag += FIRST_BONUS;
	for( int i = 0; i < Bonus::getNumTypes(); i++ )  delete m_bag[i];
	delete [] m_bag;
}

/**********************************************************/

void BonusPack::add( Bonus* newBonus )
{
	DBG( 3 ) {
		checkIDRange( newBonus->getID(), "add" );
		ASSERT( newBonus->mustBePutIntoPack(),
		        "BonusPack::add: bonus \"%s\" must not be carried in "
		        "the bonus pack\n", newBonus->getIDString() );
	}

	if( m_bag[newBonus->getID()] == NULL ) {
		m_bag[newBonus->getID()] = newBonus;
	} else {
		m_bag[newBonus->getID()]->add( newBonus );
		delete newBonus;
	}
}

/**********************************************************/

void BonusPack::remove( const int bonusID )
{
	DBG(2) checkIDRange( bonusID, "remove" );

	delete m_bag[bonusID];
	m_bag[bonusID] = NULL;
}

/**********************************************************/

void BonusPack::update( Avatar *const avatar )
{
	for( int id = FIRST_BONUS; id <= LAST_BONUS; id++ ) {
		if( m_bag[id] != NULL ) {
			if( ! m_bag[id]->updateInBonusPack(avatar) ) {
				remove( id );
			}
		}
	}
}

/**********************************************************/

void BonusPack::resetForRespawn()
{
	for( int id = FIRST_BONUS; id <= LAST_BONUS; id++ ) {
		if( m_bag[id] && !m_bag[id]->persistentInPack() ) {
			remove( id );
		}
	}
}

/**********************************************************/

Uint32 BonusPack::getSerializeBufferSize() const
{
	Uint32 size = (Uint32)Bonus::getNumTypes() *
	              Serialize<Bonus*>::sizeOf();

	ADD_TAG_SIZE( size );

	for( int id = FIRST_BONUS; id <= LAST_BONUS; id++ ) {
		if( m_bag[id] ) {
			size += m_bag[id]->getSerializeBufferSize();
			// adds the tag size to "size", if tags are used
			ADD_TAG_SIZE( size );
		}
	}

	return size;
}

/**********************************************************/

void BonusPack::serialize( Uint8*& bufferPointer ) const
{
	// debugging macro
	START_SERIALIZED_SIZE_CHECK( bufferPointer );
	SERIALIZE_TAG( bufferPointer );

	Serialize<Bonus*>::serialize( Bonus::getNumTypes(),
	                              m_bag + FIRST_BONUS,
	                              bufferPointer );

	for( int id = FIRST_BONUS; id <= LAST_BONUS; ++id ) {
		if( m_bag[id] ) {
			m_bag[id]->serialize( bufferPointer );
			SERIALIZE_TAG( bufferPointer );
		}
	}
	
	// debugging macro
	END_SERIALIZED_SIZE_CHECK( bufferPointer, BonusPack );
}

/**********************************************************/

void BonusPack::deserialize( Uint8*& bufferPointer )
{
	DESERIALIZE_TAG( bufferPointer );
	
	Serialize<Bonus*>::deserialize( Bonus::getNumTypes(),
	                                bufferPointer,
	                                m_bag + FIRST_BONUS );

	for( int id = FIRST_BONUS; id <= LAST_BONUS; ++id ) {
		if( m_bag[id] ) {
			// let the world create the bonus object, ...
			m_bag[id] = dynamic_cast<Bonus*>( m_worldPointer->newObject( id ));
			// ... but accroach it immediately from the world's control.
			m_worldPointer->unhookObject( m_bag[id] );
			m_bag[id]->deserialize( bufferPointer );
			DESERIALIZE_TAG( bufferPointer );
		}
	}
}

/**********************************************************/

void BonusPack::checkIDRange( const int         ID,
                              const char* const fn ) const
{
	 ASSERT( FIRST_BONUS <= ID && ID <= LAST_BONUS,
	         "BonusPack::ceckIDRange: called from BonusPack::"
	         "%s: %d is out of range of valid bonus IDs [%d,%d]",
	         fn, ID, ID, FIRST_BONUS, LAST_BONUS );
}

/**********************************************************/
