/* $Id: weaponhook.cpp,v 1.14 2005/12/04 17:15:24 pohlt Exp $ */

#include "weaponhook.hpp"
#include "world.hpp"
#include "avatar.hpp"
#include "rope.hpp"
#include "stationarygun.hpp"

bool WeaponHook::reallyShoot( World* world, Avatar* avatar,
                              const StationaryGun* gun ) const {
	CollidableObject* hitObject;

	if( gun ) {
		// place hook 1
		Hook* const hook1 = dynamic_cast<Hook*>( world->newObject( HOOK ));
		Vector offsetVector( -gun->getAimingDY(), gun->getAimingDX() );
		offsetVector.normalize();

		if( hook1->initialize(
		        gun->getShootingPoint() - offsetVector*2,
		        gun->getRandomizedAimingVector( 3 )*(25.0 - 4.0 * world->getRandom().getNormedReal()) -
		        offsetVector*2, hitObject ) == false ) return false;

		// place hook 2
		Hook* const hook2 = dynamic_cast<Hook*>( world->newObject( HOOK ));

		if( hook2->initialize(
		        gun->getShootingPoint() + offsetVector*2,
		        gun->getRandomizedAimingVector( 3 )*(25.0 - 4.0 * world->getRandom().getNormedReal()) +
		        offsetVector*2, hitObject ) == false ) {
			world->deleteObject( hook1 ); // ok
			return false;
		}

		// create rope
		Rope* const rope = dynamic_cast<Rope*>( world->newObject( ROPE ));
		// attach rope to shooting avatar and hook
		rope->initialize( dynamic_cast<AttachableObject*>( hook1 ),
		                  dynamic_cast<AttachableObject*>( hook2 ),
		                  avatar->getPlayer()->getPlayerOrTeamID(),
		                  200, avatar->getSkinColor() );
	}
	else {
		// place hook
		Hook* const hook = dynamic_cast<Hook*>( world->newObject( HOOK ));

		if( hook->initialize(
		       avatar->placeBesideCollRect(  hook->getCollRectX() + hook->getCollRectWidth() - 1,
		                                    -hook->getCollRectX(),
		                                     hook->getCollRectY() + hook->getCollRectHeight() - 1,
		                                    -hook->getCollRectY() ),
		       avatar->getRandomizedAimingVector( 3 )*(28.0 - 5.0 * world->getRandom().getNormedReal()) +
		       avatar->getVel(), hitObject ) == false ) {
			if( hitObject == NULL ) return false;
			// else hook could not be placed but an object was hit directly, so continue
		}
		else {
			// hook could be placed, so set hitObject to hook
			hitObject = hook;
		}

		// create rope
		Rope* const rope = dynamic_cast<Rope*>( world->newObject( ROPE ));
		// attach rope to shooting avatar and hitObject
		rope->initialize( dynamic_cast<AttachableObject*>( avatar ),
		                  dynamic_cast<AttachableObject*>( hitObject ),
		                  avatar->getPlayer()->getPlayerOrTeamID(),
		                  200, avatar->getSkinColor() );
	}

	return true;
}
